@@ -33,10 +33,6 @@ from pyhelp.output import HelpOutput
 
 
 FNAME_CONN_TABLES = 'connect_table.json'
-INPUT_PRECIP_FNAME = 'precip_input_data.csv'
-INPUT_AIRTEMP_FNAME = 'airtemp_input_data.csv'
-INPUT_SOLRAD_FNAME = 'solrad_input_data.csv'
-INPUT_GRID_FNAME = 'input_grid.csv'
 
 
 class HelpManager(object):
@@ -44,17 +40,39 @@ class HelpManager(object):
     The :class:`~pyhelp.HelpManager` is a class whose main purpose
     is to evaluate the component of the hydrologic water budget at the
     regional scale with the HELP model.
+
+    Parameters
+    ----------
+    workdir : str
+        The path to the working directory.
+    path_to_grid : str
+        The path to the csv file that contains the geomatic data, surface
+        conditions, and soil and design data for each cell of the grid
+        dividing the study area. The default is None.
+    path_to_precip : str
+        The path to the csv file that contains the input data for the
+        daily total precipitation.
+    path_to_airtemp : str
+        The path to the csv file that contains the input data for the
+        daily average air temperature
+    path_to_solrad : str
+        The path to the csv file that contains the input data for the
+        daily global solar radiation.
     """
 
-    def __init__(self, workdir):
+    def __init__(self, workdir: str, path_to_grid: str,
+                 path_to_precip: str, path_to_airtemp: str,
+                 path_to_solrad: str):
         super().__init__()
         self.grid = None
         self.precip_data = None
         self.airtemp_data = None
         self.solrad_data = None
 
-        self._workdir = None
         self.set_workdir(workdir)
+        self.load_input_grid(path_to_grid)
+        self.load_weather_input_data(
+            path_to_precip, path_to_airtemp, path_to_solrad)
 
         self._load_connect_tables()
 
@@ -83,14 +101,10 @@ class HelpManager(object):
 
     def set_workdir(self, dirname):
         """Set the working directory of the manager."""
-        if self.workdir is not None and osp.samefile(self.workdir, dirname):
-            return
+        self._workdir = dirname
         if not osp.exists(dirname):
             os.makedirs(dirname)
         os.chdir(dirname)
-        self._workdir = dirname
-        self.load_input_grid()
-        self.load_weather_input_data()
 
     # ---- Connect tables
     @property
@@ -112,41 +126,55 @@ class HelpManager(object):
                       separators=(",", ": "), ensure_ascii=False)
 
     # ---- Grid and Input
-    def load_input_grid(self):
+    def load_input_grid(self, path_to_grid: str):
         """
         Load input grid data.
 
-        Load the grid containing the geomatic data, surface conditions, and
-        soil and design data for each cell of the grid dividing the study area.
-        By default, the input grid data file must be saved in the working
-        directory and named :file:`input_grid.csv`.
+        Parameters
+        ----------
+        path_to_grid : str
+            The path to the csv file that contains the geomatic data, surface
+            conditions, and soil and design data for each cell of the grid
+            dividing the study area.
         """
-        print('Reading grid data from input csv file...')
-        grid_fname = osp.join(self.workdir, INPUT_GRID_FNAME)
-        if not osp.exists(grid_fname):
+        self.grid_filename = osp.abspath(path_to_grid)
+        print(f'Reading grid data from {path_to_grid}...')
+        if not osp.exists(path_to_grid):
             self.grid = None
             print("Grid input csv file does not exist.")
         else:
-            self.grid = load_grid_from_csv(grid_fname)
+            self.grid = load_grid_from_csv(path_to_grid)
             print('Grid data read successfully from input csv file.')
 
-    def load_weather_input_data(self):
+    def load_weather_input_data(self, path_to_precip: str,
+                                path_to_airtemp: str,
+                                path_to_solrad: str):
         """
         Load input weather data.
 
-        Load the daily precipitation, average air temperature, and
-        global solar radiation from the input weather data files. By default,
-        those files must be saved in the working directory and named,
-        respectively, :file:`precip_input_data.csv`,
-        :file:`airtemp_input_data.csv`, and :file:`solrad_input_data.csv`.
+        Parameters
+        ----------
+        path_to_precip : str
+            The path to the csv file that contains the input data for the
+            daily total precipitation.
+        path_to_airtemp : str
+            The path to the csv file that contains the input data for the
+            daily average air temperature
+        path_to_solrad : str
+            The path to the csv file that contains the input data for the
+            daily global solar radiation.
         """
-        print('Reading input weather data from csv...')
-        self.precip_data = load_weather_from_csv(
-            osp.join(self.workdir, INPUT_PRECIP_FNAME))
-        self.airtemp_data = load_weather_from_csv(
-            osp.join(self.workdir, INPUT_AIRTEMP_FNAME))
-        self.solrad_data = load_weather_from_csv(
-            osp.join(self.workdir, INPUT_SOLRAD_FNAME))
+        print(f'Reading input precip data from {path_to_precip}...')
+        self.precip_data = load_weather_from_csv(path_to_precip)
+        self.precip_filename = path_to_precip
+
+        print(f'Reading input airtemp data from {path_to_airtemp}...')
+        self.airtemp_data = load_weather_from_csv(path_to_airtemp)
+        self.airtemp_filename = path_to_airtemp
+
+        print(f'Reading input solrad data from {path_to_solrad}...')
+        self.solrad_data = load_weather_from_csv(path_to_solrad)
+        self.solrad_filename = path_to_solrad
 
         datasets = [self.precip_data, self.airtemp_data, self.solrad_data]
         datasets_name = {
@@ -203,18 +231,37 @@ class HelpManager(object):
         delete_folder_recursively(self.inputdir)
         print('done')
 
-    def build_help_input_files(self, sf_edepth=1, sf_ulai=1):
+    def build_help_input_files(self, cellnames: list = None,
+                               sf_edepth: float = 1, sf_ulai: float = 1,
+                               sf_cn: float = 1):
         """
         Clear all cached HELP input data files and generate new ones from the
         weather and grid input data files.
+
+        Parameters
+        ----------
+        cellnames : list, optional
+            The list of cell ids for which D10 and D11 HELP input files
+            are to be generated. If None, D10 and D11 HELP input files are
+            generated for each cell of the grid with a "run" value of 1.
+        sf_edepth : float, optional
+            Global scale factor for the Evaporative Zone Depth (applied to
+            the whole grid). The default is 1.
+        sf_ulai : float, optional
+            Global scale factor for the Maximum Leaf Area Index (applied to
+            the whole grid). The default is 1.
+        sf_cn : float, optional
+            Global scale factor for the Curve Number (applied to
+            the whole grid). The default is 1.
         """
         self.clear_cache()
-        self._generate_d10d11_input_files(sf_edepth=sf_edepth,
-                                          sf_ulai=sf_ulai)
-        self._generate_d4d7d13_input_files()
+        self._generate_d10d11_input_files(
+            cellnames, sf_edepth, sf_ulai, sf_cn)
+        self._generate_d4d7d13_input_files(
+            cellnames)
 
-    def _generate_d10d11_input_files(self, cellnames=None, sf_edepth=1,
-                                     sf_ulai=1):
+    def _generate_d10d11_input_files(self, cellnames, sf_edepth,
+                                     sf_ulai, sf_cn):
         """
         Prepare the D10 and D11 input datafiles for each cell.
 
@@ -223,6 +270,13 @@ class HelpManager(object):
 
         See https://github.com/cgq-qgc/pyhelp/wiki/HELP-input-files-format-description
         """
+        if sf_edepth < 0:
+            raise ValueError("sf_edepth value cannot be lower than 0.")
+        if sf_ulai < 0:
+            raise ValueError("sf_ulai value cannot be lower than 0.")
+        if sf_cn < 0:
+            raise ValueError("sf_cn value cannot be lower than 0.")
+
         d10d11_inputdir = osp.join(self.inputdir, 'd10d11_input_files')
         if not osp.exists(d10d11_inputdir):
             os.makedirs(d10d11_inputdir)
@@ -231,9 +285,14 @@ class HelpManager(object):
         # don't need the D10 or D11 input files for those that aren't.
         cellnames = self.get_run_cellnames(cellnames)
 
+        # Apply scaling factors to the grid.
+        grid = self.grid.copy()
+        grid['EZD'] = grid['EZD'] * sf_edepth
+        grid['LAI'] = grid['LAI'] * sf_ulai
+        grid['CN'] = grid['CN'] * sf_cn
+
         # Format the data from the input grid.
-        d10data, d11data = format_d10d11_inputs(
-            self.grid, cellnames, sf_edepth, sf_ulai)
+        d10data, d11data = format_d10d11_inputs(grid, cellnames)
 
         # Write the D10 and D11 input files.
         d10_conn_tbl, d11_conn_tbl = write_d10d11_allcells(
@@ -246,7 +305,7 @@ class HelpManager(object):
         self._save_connect_tables()
         print("done")
 
-    def _generate_d4d7d13_input_files(self, cellnames: list = None):
+    def _generate_d4d7d13_input_files(self, cellnames):
         """
         Generate the D4, D7, and D13 HELP input datafiles for each cell.
 
@@ -254,9 +313,6 @@ class HelpManager(object):
         D7: Mean air temperature.
         D13: Solar radiation.
         """
-        if self.grid is None:
-            return
-
         cellnames = self.cellnames if cellnames is None else cellnames
         grid_lat, grid_lon = self.get_latlon_for_cellnames(cellnames)
 
@@ -315,15 +371,40 @@ class HelpManager(object):
         self._save_connect_tables()
         print('done')
 
-    def calc_help_cells(self, path_to_hdf5=None, cellnames=None, tfsoil=0):
+    def calc_help_cells(self, path_to_hdf5=None, cellnames=None, tfsoil=0,
+                        sf_edepth: float = 1, sf_ulai: float = 1,
+                        sf_cn: float = 1,
+                        build_help_input_files: bool = True) -> HelpOutput:
         """
         Calcul the water budget for all eligible cells with HELP.
 
         Run HELP to compute the monthly water budget for the cells listed in
-        _cellnames_. Return a dict containing the resulting monthly values as
-        numpy arrays. If a file name is provided in _path_outfile_, the results
+        "cellnames". If a file name is provided in _path_outfile_, the results
         are also saved to disk in a HDF5 file.
+
+        Parameters
+        ----------
+        path_to_hdf5: str, optional
+            File path where to save the results to disk in a HDF5 file.
+        tfsoil: float, optional
+            The average air temperature, in Celcius degrees,
+            below which the soil is assumed to be freezing. The default is 0.
+        sf_edepth : float, optional
+            Global scale factor for the Evaporative Zone Depth (applied to
+            the whole grid). The default is 1.
+        sf_ulai : float, optional
+            Global scale factor for the Maximum Leaf Area Index (applied to
+            the whole grid). The default is 1.
+        sf_cn : float, optional
+            Global scale factor for the Curve Number (applied to
+            the whole grid). The default is 1.
+        build_help_input_files: bool
+            A flag to indicate whether to generate the basic HELP input
+            files before running the simulation.
         """
+        if build_help_input_files:
+            self.build_help_input_files(cellnames, sf_edepth, sf_ulai, sf_cn)
+
         # Convert from Celcius to Farenheight
         tfsoil = (tfsoil * 1.8) + 32
 
@@ -557,9 +638,6 @@ def load_weather_from_csv(filename: str) -> pd.DataFrame:
     to the dates and the columns to latitudes and longitudes of each
     data series.
     """
-    if not osp.exists(filename):
-        return None
-
     with open(filename, 'r') as csvfile:
         reader = list(csv.reader(csvfile, delimiter=','))
 
